#' @keywords internal
#' @aliases sf-package
#' @references 
#' Pebesma, E. and Bivand, R. (2023). Spatial Data
#'   Science: With Applications in R. Chapman and Hall/CRC.
#'   \doi{10.1201/9780429459016} which is also found freely
#'   online at <https://r-spatial.org/book/>
#'   
#' Pebesma, E., 2018. Simple Features for R: Standardized Support
#'   for Spatial Vector Data. The R Journal 10 (1), 439-446,
#'   \doi{10.32614/RJ-2018-009} (open access)
"_PACKAGE"
