% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-transformers.R
\name{internal}
\alias{internal}
\alias{.stop_geos}
\title{Internal functions}
\usage{
.stop_geos(msg)
}
\arguments{
\item{msg}{error message}
}
\description{
Internal functions
}
